package com.persagy.apm.energy.report.eer.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.centermiddleware.model.vo.ProjectEERDataVo;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.common.utils.GraphDataUtils;
import com.persagy.apm.energy.report.eer.client.EerClient;
import com.persagy.apm.energy.report.eer.enums.EnergyEfficiencyTimeTypeEnum;
import com.persagy.apm.energy.report.eer.vo.EnergyEfficiencyDataListVO;
import com.persagy.apm.energy.report.eer.vo.EnergyEfficiencyDataVO;
import com.persagy.apm.energy.report.eer.vo.QueryEnergyEfficiencyDataDTO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.DateGraphItemVO;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Service
@Slf4j
public class EerServiceImp {
    @Autowired
    private EerClient eerClient;

    @Value("${group.code}")
    String groupCode;


    /**
     * 从冷站能耗服务获取指标的取值
     *
     * @return {时间：取值}
     * @author lixing 2022/4/13 16:31
     */
    @SneakyThrows
    private Map<String, Double> getTargetDataFromEnergyEfficiency(
            QueryEnergyEfficiencyDataDTO queryEnergyEfficiencyDataDTO) {
        Map<String, Double> dataMap = new HashMap<String, Double>();
        PoemsFeignResponse<EnergyEfficiencyDataListVO> efficiencyDataListResponse = eerClient.
                queryProjectTargetDataList(groupCode, JSON.toJSONString(queryEnergyEfficiencyDataDTO));

        // 解决返回结果，获取月能效值
        List<EnergyEfficiencyDataVO> dataList = analysisResponse(efficiencyDataListResponse);
        if (CollectionUtils.isEmpty(dataList)) {
            return dataMap;
        }

        dataList.forEach(eer -> {
            Double value = eer.getData() == null ? null : Double.valueOf(eer.getData());
            dataMap.put(eer.getTime(), value);
        });

        return dataMap;
    }


    public List<ProjectEERDataVo> getProjectEerDataList(List<String> pjIdList, String groupCode, Date reportDate) {
        log.info("项目列表:" + pjIdList + "查询EER数据开始");
        if (CollectionUtils.isEmpty(pjIdList) || StringUtils.isEmpty(groupCode) || null == reportDate) {
            log.info("查询ProjectEERDataVo时参数为空，故不进行查询");
            return new ArrayList<>();
        }
        List<ProjectEERDataVo> resultList = new ArrayList<>();
        try {
            for (String pjId : pjIdList) {
                ProjectEERDataVo projectEERDataVo = new ProjectEERDataVo();
                Date endTime = DateUtils.getMonthOff(reportDate, 1);
                List<Date> dateList = DateUtils.getDayList(reportDate, endTime);
                // 获取每天的eer数据
                Map<String, Double> eerDayDataMap = getTargetDataFromEnergyEfficiency(
                        initEerDayQueryDTO(pjId, reportDate, endTime));
                // 获取当月的eer数据
                Map<String, Double> eerMonthDataMap = getTargetDataFromEnergyEfficiency(
                        initEerMonthQueryDTO(pjId, reportDate, endTime));
                // 获取当月的制冷量数据
                Map<String, Double> coolMonthDataMap = getTargetDataFromEnergyEfficiency(
                        initCoolMonthQueryDTO(pjId, reportDate, endTime));
                List<DateGraphItemVO> dateGraphItemVOList = new ArrayList<>();
                for (Date date : dateList) {
                    String dateStr = DateUtils.date2Str(date, DateUtils.SDF_SECOND);
                    DateGraphItemVO dateGraphItemVO = new DateGraphItemVO();
                    dateGraphItemVO.setDate(date);
                    Double eer = eerDayDataMap.get(dateStr);
                    dateGraphItemVO.setValue(eer);
                    dateGraphItemVOList.add(dateGraphItemVO);
                }
                String reportDateStr = DateUtils.date2Str(reportDate, DateUtils.SDF_SECOND);
                Double allEer = eerMonthDataMap.get(reportDateStr);
                projectEERDataVo.setProjectId(pjId);
                projectEERDataVo.setEer(allEer);
                projectEERDataVo.setCoolingCapacityCurrentMonth(coolMonthDataMap.get(reportDateStr));
                projectEERDataVo.setEerDetailInfo(dateGraphItemVOList);
                projectEERDataVo.setMaxEerTimeList(GraphDataUtils.getMaxOrMinDateList(dateGraphItemVOList, -1));
                projectEERDataVo.setMinEerTimeList(GraphDataUtils.getMaxOrMinDateList(dateGraphItemVOList, 1));
                resultList.add(projectEERDataVo);
            }
        } catch (Exception e) {
            log.error("计算ProjectEERDataVo异常:", e);
        }
        log.info("项目列表:" + pjIdList + "查询EER数据结束");
        return resultList;
    }

    /**
     * 初始化能效分析查询条件
     *
     * @param projectId 项目id
     * @param targetId 能效指标编码
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param timeType 时间维度 参考EnergyEfficiencyTimeTypeEnum
     * @return 能效分析查询条件
     * @author lixing 2022/4/26 16:50
     */
    private QueryEnergyEfficiencyDataDTO initQueryDTO(
            String projectId, String targetId, Date startTime, Date endTime, Integer timeType) {

        QueryEnergyEfficiencyDataDTO queryEnergyEfficiencyDataDTO = new QueryEnergyEfficiencyDataDTO();
        queryEnergyEfficiencyDataDTO.setTargetId(targetId);
        queryEnergyEfficiencyDataDTO.setProjectId(projectId);
        queryEnergyEfficiencyDataDTO.setTimeFrom(DateUtils.date2Str(startTime, DateUtils.SDF_SECOND));
        queryEnergyEfficiencyDataDTO.setTimeTo(DateUtils.date2Str(endTime, DateUtils.SDF_SECOND));
        queryEnergyEfficiencyDataDTO.setTimeType(timeType);

        return queryEnergyEfficiencyDataDTO;
    }

    /**
     * 初始化eer查询条件
     *
     * @param projectId 项目id
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param timeType 时间维度 参考EnergyEfficiencyTimeTypeEnum
     * @return eer查询条件
     * @author lixing 2022/4/26 16:52
     */
    private QueryEnergyEfficiencyDataDTO initEerQueryDTO(String projectId, Date startTime, Date endTime, Integer timeType) {
        // 冷站eer编码
        String targetId = "SysCOP";
        return initQueryDTO(projectId, targetId, startTime, endTime, timeType);
    }

    /**
     * 初始化eer日数据查询条件
     *
     * @param projectId 项目id
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return eer日数据查询条件
     * @author lixing 2022/4/26 16:52
     */
    private QueryEnergyEfficiencyDataDTO initEerDayQueryDTO(String projectId, Date startTime, Date endTime) {
        return initEerQueryDTO(projectId, startTime, endTime, EnergyEfficiencyTimeTypeEnum.DAY.getType());
    }

    /**
     * 初始化eer月数据查询条件
     *
     * @param projectId 项目id
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return eer月数据查询条件
     * @author lixing 2022/4/26 16:52
     */
    private QueryEnergyEfficiencyDataDTO initEerMonthQueryDTO(String projectId, Date startTime, Date endTime) {
        return initEerQueryDTO(projectId, startTime, endTime, EnergyEfficiencyTimeTypeEnum.MONTH.getType());
    }

    private QueryEnergyEfficiencyDataDTO initCoolMonthQueryDTO(String projectId, Date startTime, Date endTime) {
        String targetId = "CoolP";
        return initQueryDTO(projectId, targetId, startTime, endTime, EnergyEfficiencyTimeTypeEnum.MONTH.getType());
    }

    private List<EnergyEfficiencyDataVO> analysisResponse(
            PoemsFeignResponse<EnergyEfficiencyDataListVO> efficiencyDataListResponse) {
        // 解决返回结果，获取月能效值
        if (efficiencyDataListResponse == null) {
            return null;
        }
        List<EnergyEfficiencyDataListVO> efficiencyDataList = efficiencyDataListResponse.getContent();
        if (CollectionUtils.isEmpty(efficiencyDataList)) {
            return null;
        }
        EnergyEfficiencyDataListVO energyEfficiencyDataListVO = efficiencyDataList.get(0);
        if (energyEfficiencyDataListVO == null) {
            return null;
        }
        return energyEfficiencyDataListVO.getList();
    }

}
